import 'loading.dart';
import 'import_base_ui_lib.dart';
export 'import_base_ui_lib.dart';

class BasePage extends StatefulWidget {
  final BasePageState basePageState;
  const BasePage(this.basePageState, {Key? key}):super(key: key);
  @override
  BasePageState createState() => basePageState;
}

abstract class BasePageState extends State<BasePage> {
  BaseBloc? bloc;

  @override
  @mustCallSuper
  dispose() {
    if (bloc != null && !bloc!.isClosed) bloc?.close();
    super.dispose();
  }

  @override
  @mustCallSuper
  initState() {
    initBloc();
    super.initState();
    initUI();
  }

  @override
  Widget build(BuildContext context, {Color color = Colors.white}) => Scaffold(
      body: GestureDetector(
          onTapDown: (value) => clearFocus(),
          child: Stack(children: [
            Container(color: color, child: createUI(context)),
            BlocBuilder<BaseBloc, BaseState>(
                bloc: bloc,
                builder: (context, state) =>
                state.showLoading ? const Loading() : const SizedBox())
          ])));

  @protected
  Widget createUI(BuildContext context);

  @protected
  void initBloc();

  @protected
  void initUI();

  void clearFocus() {
    final FocusScopeNode currentScope = FocusScope.of(context);
    if (!currentScope.hasPrimaryFocus && currentScope.hasFocus) FocusManager.instance.primaryFocus?.unfocus();
  }

  bool checkResponse(BaseResponse response, {bool showError = true, bool passString = false}) =>
      CoreUtilUI.checkResponse(context, response, logout, showError: showError, passString: passString);

  void back({dynamic value = false}) => CoreUtilUI.goBack(context, value);

  void showLogin(dynamic page) => CoreUtilUI.showCustomAlertDialog(context,
      MultiLanguage.get(LanguageKey.msgNotLogin), isActionCancel: true).then((value) {
    if (value != null && value) CoreUtilUI.goToPage(context, page, clearAllPage: true);
  });

  void logout(BuildContext context);
}
